%% main function

function Phi=chan_vese(R)
    close all
    I       = double(imread('image.pgm'));
    [m,n]   = size(I);
    center  = floor([m/2 n/2]);
    [X, Y]  = meshgrid(1:m,1:n);
    
    % some interesting values for R: 10, 100, 150
    if nargin == 0
        R = 100;
    end

    Phi     = R - sqrt((X-center(1)).^2 + (Y-center(2)).^2);
    
    figure, colormap gray
    loop = true;
    for i=1:500
        for k=1:20
            dPhi = update(Phi,I);
            Phi= Phi+0.001*dPhi;
        end
        Phi = redistance(Phi);
        imagesc(Phi>0)
        title(['Iteration ',num2str(i)])
        axis equal
        drawnow
    end
    return;

%% helper functions

function [mu1,mu2] = approxRegions(Phi,I)
    mu1 = mean(I(Phi>0));
    mu2 = mean(I(Phi<0));
return;


function dPhi = update(Phi,I)
    eps   = 1e-8;
    nu    = 50000;

    Phix  = dxp(Phi);
    Phiy  = dyp(Phi);

    scale = 1./sqrt(Phix.^2+Phiy.^2+eps);
    Phix  = Phix.*scale;
    Phiy  = Phiy.*scale;

    Phixx =dxm(Phix);
    Phiyy =dym(Phiy);
    [mu1, mu2] = approxRegions(Phi,I);
    dPhi = nu*(Phixx+Phiyy)-(I-mu1).^2+(I-mu2).^2;
    dPhi = dPhi.*dirac(Phi);
return;


% derivative of atan(x/eps)
function delta = dirac(Phi)
    eps   = 3;
    delta = eps./(pi*(eps*eps+Phi.^2));
return;


function sdf = redistance(Phi)
    sdf = bwdist(Phi<0) - bwdist(Phi>0);
return;